# Generated from winval.g4 by ANTLR 4.10.1
from antlr4 import *
if __name__ is not None and "." in __name__:
    from .winvalParser import winvalParser
else:
    from winvalParser import winvalParser

# This class defines a complete listener for a parse tree produced by winvalParser.
class winvalListener(ParseTreeListener):

    # Enter a parse tree produced by winvalParser#constraint.
    def enterConstraint(self, ctx:winvalParser.ConstraintContext):
        pass

    # Exit a parse tree produced by winvalParser#constraint.
    def exitConstraint(self, ctx:winvalParser.ConstraintContext):
        pass


    # Enter a parse tree produced by winvalParser#AccessExpr.
    def enterAccessExpr(self, ctx:winvalParser.AccessExprContext):
        pass

    # Exit a parse tree produced by winvalParser#AccessExpr.
    def exitAccessExpr(self, ctx:winvalParser.AccessExprContext):
        pass


    # Enter a parse tree produced by winvalParser#StringExpr.
    def enterStringExpr(self, ctx:winvalParser.StringExprContext):
        pass

    # Exit a parse tree produced by winvalParser#StringExpr.
    def exitStringExpr(self, ctx:winvalParser.StringExprContext):
        pass


    # Enter a parse tree produced by winvalParser#BoolExpr.
    def enterBoolExpr(self, ctx:winvalParser.BoolExprContext):
        pass

    # Exit a parse tree produced by winvalParser#BoolExpr.
    def exitBoolExpr(self, ctx:winvalParser.BoolExprContext):
        pass


    # Enter a parse tree produced by winvalParser#FloatExpr.
    def enterFloatExpr(self, ctx:winvalParser.FloatExprContext):
        pass

    # Exit a parse tree produced by winvalParser#FloatExpr.
    def exitFloatExpr(self, ctx:winvalParser.FloatExprContext):
        pass


    # Enter a parse tree produced by winvalParser#SetExpr.
    def enterSetExpr(self, ctx:winvalParser.SetExprContext):
        pass

    # Exit a parse tree produced by winvalParser#SetExpr.
    def exitSetExpr(self, ctx:winvalParser.SetExprContext):
        pass


    # Enter a parse tree produced by winvalParser#VariableExpr.
    def enterVariableExpr(self, ctx:winvalParser.VariableExprContext):
        pass

    # Exit a parse tree produced by winvalParser#VariableExpr.
    def exitVariableExpr(self, ctx:winvalParser.VariableExprContext):
        pass


    # Enter a parse tree produced by winvalParser#IntExpr.
    def enterIntExpr(self, ctx:winvalParser.IntExprContext):
        pass

    # Exit a parse tree produced by winvalParser#IntExpr.
    def exitIntExpr(self, ctx:winvalParser.IntExprContext):
        pass


    # Enter a parse tree produced by winvalParser#ListExpr.
    def enterListExpr(self, ctx:winvalParser.ListExprContext):
        pass

    # Exit a parse tree produced by winvalParser#ListExpr.
    def exitListExpr(self, ctx:winvalParser.ListExprContext):
        pass


    # Enter a parse tree produced by winvalParser#EmptySetExpr.
    def enterEmptySetExpr(self, ctx:winvalParser.EmptySetExprContext):
        pass

    # Exit a parse tree produced by winvalParser#EmptySetExpr.
    def exitEmptySetExpr(self, ctx:winvalParser.EmptySetExprContext):
        pass


    # Enter a parse tree produced by winvalParser#ParenExpr.
    def enterParenExpr(self, ctx:winvalParser.ParenExprContext):
        pass

    # Exit a parse tree produced by winvalParser#ParenExpr.
    def exitParenExpr(self, ctx:winvalParser.ParenExprContext):
        pass


    # Enter a parse tree produced by winvalParser#InfixExpr.
    def enterInfixExpr(self, ctx:winvalParser.InfixExprContext):
        pass

    # Exit a parse tree produced by winvalParser#InfixExpr.
    def exitInfixExpr(self, ctx:winvalParser.InfixExprContext):
        pass


    # Enter a parse tree produced by winvalParser#PrefixExpr.
    def enterPrefixExpr(self, ctx:winvalParser.PrefixExprContext):
        pass

    # Exit a parse tree produced by winvalParser#PrefixExpr.
    def exitPrefixExpr(self, ctx:winvalParser.PrefixExprContext):
        pass


    # Enter a parse tree produced by winvalParser#VariableExpr2.
    def enterVariableExpr2(self, ctx:winvalParser.VariableExpr2Context):
        pass

    # Exit a parse tree produced by winvalParser#VariableExpr2.
    def exitVariableExpr2(self, ctx:winvalParser.VariableExpr2Context):
        pass


    # Enter a parse tree produced by winvalParser#elements.
    def enterElements(self, ctx:winvalParser.ElementsContext):
        pass

    # Exit a parse tree produced by winvalParser#elements.
    def exitElements(self, ctx:winvalParser.ElementsContext):
        pass



del winvalParser